from skitai.saddle import Saddle
import skitai

app = Saddle (__name__)

app.debug = True
app.use_reloader = True
app.jinja_overlay ()

app.authorization = "digest"
app.authenticate = True
app.realm = "Secured Area"
app.users = {"admin": ("1111", 0, {'role': 'root'})}
app.authenticate = False

@app.route ("/", authenticate = 1)
def index (was):	
	return was.render ("index.html")

if __name__ == "__main__":
	import skitai
	skitai.run (
		mount = ("/", app)
	)
	


from skitai.saddle import Saddle
import skitai

app = Saddle (__name__)

app.debug = True
app.use_reloader = True
app.jinja_overlay ()

@app.route ("/")
def index (was):	
	return was.render ("index.html")

@app.route ("/documentation")
def documentation (was):
	req = was.get ("https://pypi.python.org/pypi/skitai")
	pypi_content = (
			"<h4>"
			"<p>It seems some problem at <a href='https://pypi.python.org/pypi/skitai'>PyPi</a>.</p>"
			"</h4>"	
			"<p>Please visit <a href='https://pypi.python.org/pypi/skitai'> https://pypi.python.org/pypi/skitai</a></p>"
		)
	
	rs = req.getwait (10)
	if rs.data	:
		content = rs.data.decode ("utf8")
		s = content.find ('<div class="section">')
		if s != -1:		
			e = content.find ('<a name="downloads">', s)
			if e != -1:						
				pypi_content = "<h4>This contents retrieved right now using skitai was service from <a href='https://pypi.python.org/pypi/skitai'> https://pypi.python.org/pypi/skitai</a></h4>" + content [s:e]
	
	return was.render ("documentation.html", content = pypi_content)

@app.route ("/hello")
def hello (was, num = 1):
	was.response ["Content-Type"] = "text/plain"
	return "\n".join (["hello" for i in range (int(num))])

@app.route ("/redirect1")
def redirect1 (was):
	return was.response ("301 Object Moved", "", headers = [("Location", "/redirect2")])

@app.route ("/redirect2")
def redirect2 (was):
	return was.response ("301 Object Moved", "", headers = [("Location", "/")])

@app.route ("/upload")
def upload (was, **karg):
	return was.response ("200 OK", str (karg), headers = [("Content-Type", "text/plain")])

@app.route ("/post")
def post (was, username):	
		return 'USER: %s' % username

	
if __name__ == "__main__":
	import skitai
	skitai.run (
		address = "0.0.0.0",
		port = 5000,
		mount = [
			("/", app),
			("/", 'static')
		]	
	)
	


from flask import Flask, render_template, request
from skitai.saddle import jinjapatch
import skitai

app = Flask(__name__)
app.debug = True
app.use_reloader = True
app.jinja_env = jinjapatch.overlay (__name__)

@app.route ("/echo")
def echo ():
	if "websocket_init" in request.environ:
		request.environ ["websocket_init"] = (skitai.WS_SIMPLE, 60, ("message", "client_id", "event"))
		return ""
	
	request.environ ["websocket"].send ('1st: ' + request.args.get ("message", ""))
	return "2nd: " + request.args.get ("message", "")

@app.route ("/chat")
def chat ():
	if "websocket_init" in request.environ:
		request.environ ["websocket_init"] = (skitai.WS_GROUPCHAT, 60, ("message", "client_id", "event", "room_id"))
		return ""

	event = request.args.get ("event")
	client_id = request.args.get ("client_id", "")
	message = request.args.get ("message", "")
	
	if event == skitai.WS_EVT_ENTER:
		return "Client %s has entered" % client_id
	if event == skitai.WS_EVT_EXIT:
		return "Client %s has leaved" % client_id
	return "Client %s Said: %s" % (client_id, message)
				
@app.route ("/")
def websocket ():
	mode = request.args.get('mode', '')
	if mode == "talk":
		mode += "?name=Hans"
	elif mode == "chat":	
		mode += "?room_id=1"
	elif mode == "multi":	
		mode += "?room_id=2"
	return render_template ("websocket-flask.html", path = mode)
	
if __name__ == "__main__":
	import skitai
	skitai.run (
		address = "0.0.0.0",
		port = 5000,
		mount = ("/websocket", app)
	)
	


from skitai.saddle import Saddle
import skitai

app = Saddle (__name__)

app.debug = True
app.use_reloader = True
app.jinja_overlay ()

@app.route ("/")
def index (was):	
	return "<h1>Skitai Proxy</h1>"

if __name__ == "__main__":
	import skitai
	skitai.run (
		mount = ("/", app),
		proxy = 1
	)
	


#!/usr/bin/python3
# 2016. 2. 29 by Hans Roh hansroh@gmail.com

import subprocess
import sys, os, getopt
from aquests.lib import flock, pathtool, logger, confparse
import signal
import time
import glob
import threading
from datetime import datetime


EXIT_CODE = None

def hTERM (signum, frame):			
	global EXIT_CODE
	EXIT_CODE = 0

def hHUP (signum, frame):			
	global EXIT_CODE
	EXIT_CODE = 3


class	CronManager:	
	def __init__ (self, config, logpath, varpath, consol):
		self.confn = config.fn
		self.logpath = logpath
		self.varpath = varpath
		self.consol = consol
		self.confmtime = 0
		self.last_maintern = 0
		self.flock = None		
		self.jobs = []
		self.currents = {}
		self.setup ()		
	
	def maintern_shutdown_request (self, now):
		global EXIT_CODE
		
		req = self.flock.lockread ("signal")
		if not req: return
		self.logger ("[info] got signal - %s" % req)
		if req in ("terminate", "shutdown"):
			EXIT_CODE = 0
		elif req == "restart":
			EXIT_CODE = 3	
		elif req == "rotate":
			try: self.logger.rotate ()
			except: self.logger.trace ()
		else:
			self.logger ("[error] unknown signal - %s" % req)
		self.flock.unlock ("signal")
	
	def close (self):
		self.logger ("[info] trying to kill childs...")
		self.maintern (time.time (), kill = True)
		self.logger ("[info] service cron stopped")
	
	def kill (self, pid, cmd):
		self.logger ("[info] trying to kill pid:%d %s" % (pid, cmd))			
		if os.name == "nt":
			import win32api, win32con, pywintypes
			try:
				handle = win32api.OpenProcess (win32con.PROCESS_TERMINATE, 0, pid)
				win32api.TerminateProcess (handle, 0)
				win32api.CloseHandle (handle)
				
			except pywintypes.error as why:
				if why.errno != 87:
					self.logger ("[error] failed killing job pid:%d %s" % (pid, cmd))
					
		else:
			child.kill ()			
			
	def maintern (self, current_time, kill = False):
		for pid, (cmd, started, child) in list (self.currents.items ()):			
			rcode = child.poll ()
			due = ((current_time - started) * 1000)
			
			if due > 60000:
				due -= 60000				
			if due > 120000:
				due = "%2.1f min" % (due / 1000. / 60.,)
			elif due > 1000:
				due = "%2.1f sec" % (due / 1000.,)
			else:
				due = "%d ms" % due				
			
			if rcode is None:
				if kill:
					 self.kill (pid, cmd)					 
					 del self.currents [pid]					 
				else:
					self.logger ("[info] job still running pid:%d for %s, %s" % (pid, due, cmd))
				continue
			
			self.logger ("[info] job has been finished pid:%d with rcode:%d for %s, %s" % (pid, rcode, due, cmd))
			del self.currents [pid]
		self.last_maintern = time.time ()
							
	def run (self):
		self.logger ("[info] service cron started")
		try:
			try:
				self.loop ()
			except:
				self.logger.trace ()
		finally:
			self.close ()		
	
	def parse (self, unitd, umax = 60):
		#print ("---", unitd)
		
		if unitd == "*":
			return []
						
		try: 
			unit, interval = unitd.split ("/", 1)
		except ValueError:
			unit, interval = unitd, 0
		else:
			interval = int (interval)
			if interval == 1 and unit in "0*":
				return []			
			if unit == "*":
				unit = "0"								
			if interval > umax / 2.:
				raise ValueError ("interval %d is too big" % interval)
		
		units = {}
		_all = unit.split (",")
		_all_len = len (_all)
		
		for subunit in _all:
			if subunit.find ("-") == -1:
				if interval == 0 or _all_len > 1:
					units [int (subunit)] = None
				else:	
					for i in range (int (subunit), umax, interval):
						units [i] = None
						
			else:
				a, b = subunit.split ("-", 1)
				if a == "":
					a = 0
				else:
					a = int (a)
				if b == "":
					b = umax			
				else:	
					b = int (b) + 1
				for i in range (a, b, interval == 0 and 1 or interval):
					units [i] = None
		
		# add sunday (0 or 7)
		if umax == 7 and 0 in units:
			del units [0]
			units [7] = None
			
		r = list (units.iterkeys ())
		r.sort ()
		return r
	
	def update_jobs (self):
		mtime = os.path.getmtime (self.confn)
		
		if self.confmtime != mtime:
			if self.confmtime:
				self.logger ("[error] crontab updated, refreshing...")
			self.jobs = []				
			cf = confparse.ConfParse (self.confn)
			jobs = cf.getopt ("crontab")
			if not jobs: return
			for args in jobs:
				args = args.split (" ", 5)
				if len (args) != 6:
					self.logger ("[error] invalid cron command %s" % (args,))
					continue
				
				try:
					job = (
						self.parse (args [0], 60),
						self.parse (args [1], 24),
						self.parse (args [2], 31),
						self.parse (args [3], 12),
						self.parse (args [4], 7),
						args [5]
					)					
				
				except ValueError as why:
					self.logger ("[error] %s, %s" % (args [5], why))
					continue	
					
				except:
					self.logger.trace ()
					continue	
				
				else:
					self.jobs.append (job)
					self.logger ("[info] job added %s" % str (job))
					
		now = datetime.now ().timetuple ()	
		for m, h, d, M, w, cmd in self.jobs:			
			if M and now.tm_mon not in M:
				continue
			if w and now.tm_wday + 1 not in w:
				continue
			if d and now.tm_mday not in d:
				continue	
			if h and now.tm_hour not in h:
				continue
			if m and now.tm_min not in m:
				continue
			else:
				threading.Thread (target = self.execute, args = (cmd,)).start ()
				
		self.confmtime = mtime
		
	def setup (self):
		if self.consol:
			self.logger = logger.screen_logger ()
		else:
			self.logger = logger.rotate_logger (self.logpath, "cron", "daily")
		
		if os.name == "nt":
			self.flock = flock.Lock (os.path.join (self.varpath, "lock"))
			self.flock.unlockall ()
		
		if os.name != "nt":
			def hUSR1 (signum, frame):	
				self.logger.rotate ()
			
			signal.signal(signal.SIGTERM, hTERM)
			signal.signal(signal.SIGQUIT, hTERM)
			signal.signal(signal.SIGHUP, hHUP)
		
	def execute (self, cmd):					
		if os.name == "nt":
			child = subprocess.Popen (
				cmd, 
				#shell = True, if true, can't terminate process bacuase run within cmd's child
				creationflags=subprocess.CREATE_NEW_PROCESS_GROUP
			)

		else:
			child = subprocess.Popen (
				"exec " + cmd, 
				shell = True
			)
		
		self.logger ("[info] job started with pid:%d %s" % (child.pid, cmd))
		self.currents [child.pid] = (cmd, time.time (), child)
	
	def loop (self):
		global EXIT_CODE
		while 1:
			if EXIT_CODE is not None:
				break
			now = time.time ()	
			self.update_jobs ()
			self.maintern (now)
			for i in range (60):
				if os.name == "nt" and i % 10 == 0:
					self.maintern_shutdown_request (now)
				if EXIT_CODE is not None:
					break
				time.sleep (1)
		
		
def usage ():
		print("""
Usage:
	skitaid-cron.py [options...]

Options:
	--log or -l: print log
	--verbose or -v
	--help or -h

Examples:
	ex. skitaid-cron.py -v
	ex. skitaid-cron.py -l
	""")


if __name__ == "__main__":
	argopt = getopt.getopt(sys.argv[1:], "hvl", ["help", "verbose", "log"])
	_varpath = None
	_consol = False
	_log = False
	
	for k, v in argopt [0]:
		if k == "--log" or k == "-l":
			_log = True
		elif k == "--verbose" or k == "-v":	
			_consol = True
		elif k == "--help" or k == "-h":	
			usage ()
			sys.exit ()
	
	import skitaid
	_config = skitaid.cf
	_varpath = os.path.join (skitaid.VARDIR, "daemons", "cron")
	_logpath = os.path.join (skitaid.LOGDIR, "daemons", "cron")
	
	if _log:
		skitaid.printlog (os.path.join (_logpath, "cron.log"))
		sys.exit (0)
		
	lck = flock.Lock (os.path.join (_varpath, "lock"))
	pidlock = lck.get_pidlock ()
	
	if pidlock.isalive ():
		print("[error] already running")
		sys.exit (1)
	
	pathtool.mkdir (_logpath)
	if not _consol: # service mode
		from aquests.lib import devnull		
		sys.stdout = devnull.devnull ()		
		sys.stderr = open (os.path.join (_logpath, "stderr.log"), "a")
	
	pidlock.make ()
	service = CronManager (_config, _logpath, _varpath, _consol)
		
	try:
		service.run ()
	finally:	
		pidlock.remove ()
		if not _consol:
			sys.stderr.close ()
		sys.exit (EXIT_CODE)
		




if __name__ == "__main__":
	import skitai
	import os
	
	skitai.run (
		mount = [
			("/", os.path.join (os.getcwd (), os.path.join (os.path.split (__file__)[0], "app.py"))),
			("/", 'static')
		],
		certfile = r"C:\skitaid\etc\certifications\example.pem",
		keyfile = r"C:\skitaid\etc\certifications\example.key",
		passphrase = "fatalbug"
	)



from skitai.saddle import Saddle
import time, math
import json
from apppackages import route_guide_pb2

app = Saddle (__name__)
app.debug = True
app.use_reloader = True

@app.route ("/add_number")
def RouteChat (was, a, b):
	return a + b

@app.route ("/")
def RouteChat (was, a, b):
	return "<h1>XML-RPC</h1>"
	
if __name__ == "__main__":
	import skitai
	skitai.run (
		address = "0.0.0.0",
		port = 5000,
		mount = ("/rpc2", app)
	)
	



from skitai.saddle import Saddle
import time, math
import json
from apppackages import route_guide_pb2

app = Saddle (__name__)
app.debug = True
app.use_reloader = True


def get_feature (feature_db, point):
	for feature in feature_db:
		if feature.location == point:
			return feature
	return None

def get_distance(start, end):
	coord_factor = 10000000.0
	lat_1 = start.latitude / coord_factor
	lat_2 = end.latitude / coord_factor
	lon_1 = start.longitude / coord_factor
	lon_2 = end.longitude / coord_factor
	lat_rad_1 = math.radians(lat_1)
	lat_rad_2 = math.radians(lat_2)
	delta_lat_rad = math.radians(lat_2 - lat_1)
	delta_lon_rad = math.radians(lon_2 - lon_1)

	a = (pow(math.sin(delta_lat_rad / 2), 2) +
			 (math.cos(lat_rad_1) * math.cos(lat_rad_2) *
				pow(math.sin(delta_lon_rad / 2), 2)))
	c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
	R = 6371000; # metres
	return R * c

@app.route ("/RouteChat")
def RouteChat (was, new_notes):
	prev_notes = []	
	for new_note in new_notes:
		for prev_note in prev_notes:
			if prev_note.location == new_note.location:
				yield prev_note
		prev_notes.append(new_note)	
		
@app.route ("/RecordRoute")
def RecordRoute (self, points):	
	point_count = 0
	feature_count = 0
	distance = 0.0
	prev_point = None

	start_time = time.time()
	for point in points:
		point_count += 1
		if get_feature(db, point):
			feature_count += 1
		if prev_point:
			distance += get_distance(prev_point, point)
		prev_point = point

	elapsed_time = time.time() - start_time
	return route_guide_pb2.RouteSummary(
		point_count=point_count,
		feature_count=feature_count,
		distance=int(distance),
		elapsed_time=int(elapsed_time)
	)
		
@app.route ("/GetFeature")
def GetFeature (was, point):
	feature = get_feature(db, point)
	if feature is None:
		return route_guide_pb2.Feature(name="", location=point)
	else:
		return feature

@app.route ("/ListFeatures")
def ListFeatures (was, rectangle):
	left = min(rectangle.lo.longitude, rectangle.hi.longitude)
	right = max(rectangle.lo.longitude, rectangle.hi.longitude)
	top = max(rectangle.lo.latitude, rectangle.hi.latitude)
	bottom = min(rectangle.lo.latitude, rectangle.hi.latitude)
	for feature in db:
		if (feature.location.longitude >= left and feature.location.longitude <= right and feature.location.latitude >= bottom and feature.location.latitude <= top):
			yield feature
	
@app.route ("/test")
def test (was):	
	stub = was.grpc ("http://127.0.0.1:5000/routeguide.RouteGuide")
	point = route_guide_pb2.Point (latitude=409146138, longitude=-746188906)
	feature = stub.GetFeature (point)
	rs = feature.getwait ()
	return str (rs.data)
	

@app.route ("/")
def index (was):	
	return "<h1>Route Guide<h1>"
	
_jsondb = """	
[{
		"location": {
				"latitude": 407838351,
				"longitude": -746143763
		},
		"name": "Patriots Path, Mendham, NJ 07945, USA"
}, {
		"location": {
				"latitude": 408122808,
				"longitude": -743999179
		},
		"name": "101 New Jersey 10, Whippany, NJ 07981, USA"
}, {
		"location": {
				"latitude": 413628156,
				"longitude": -749015468
		},
		"name": "U.S. 6, Shohola, PA 18458, USA"
}, {
		"location": {
				"latitude": 419999544,
				"longitude": -740371136
		},
		"name": "5 Conners Road, Kingston, NY 12401, USA"
}, {
		"location": {
				"latitude": 414008389,
				"longitude": -743951297
		},
		"name": "Mid Hudson Psychiatric Center, New Hampton, NY 10958, USA"
}, {
		"location": {
				"latitude": 419611318,
				"longitude": -746524769
		},
		"name": "287 Flugertown Road, Livingston Manor, NY 12758, USA"
}, {
		"location": {
				"latitude": 406109563,
				"longitude": -742186778
		},
		"name": "4001 Tremley Point Road, Linden, NJ 07036, USA"
}, {
		"location": {
				"latitude": 416802456,
				"longitude": -742370183
		},
		"name": "352 South Mountain Road, Wallkill, NY 12589, USA"
}, {
		"location": {
				"latitude": 412950425,
				"longitude": -741077389
		},
		"name": "Bailey Turn Road, Harriman, NY 10926, USA"
}, {
		"location": {
				"latitude": 412144655,
				"longitude": -743949739
		},
		"name": "193-199 Wawayanda Road, Hewitt, NJ 07421, USA"
}, {
		"location": {
				"latitude": 415736605,
				"longitude": -742847522
		},
		"name": "406-496 Ward Avenue, Pine Bush, NY 12566, USA"
}, {
		"location": {
				"latitude": 413843930,
				"longitude": -740501726
		},
		"name": "162 Merrill Road, Highland Mills, NY 10930, USA"
}, {
		"location": {
				"latitude": 410873075,
				"longitude": -744459023
		},
		"name": "Clinton Road, West Milford, NJ 07480, USA"
}, {
		"location": {
				"latitude": 412346009,
				"longitude": -744026814
		},
		"name": "16 Old Brook Lane, Warwick, NY 10990, USA"
}, {
		"location": {
				"latitude": 402948455,
				"longitude": -747903913
		},
		"name": "3 Drake Lane, Pennington, NJ 08534, USA"
}, {
		"location": {
				"latitude": 406337092,
				"longitude": -740122226
		},
		"name": "6324 8th Avenue, Brooklyn, NY 11220, USA"
}, {
		"location": {
				"latitude": 406421967,
				"longitude": -747727624
		},
		"name": "1 Merck Access Road, Whitehouse Station, NJ 08889, USA"
}, {
		"location": {
				"latitude": 416318082,
				"longitude": -749677716
		},
		"name": "78-98 Schalck Road, Narrowsburg, NY 12764, USA"
}, {
		"location": {
				"latitude": 415301720,
				"longitude": -748416257
		},
		"name": "282 Lakeview Drive Road, Highland Lake, NY 12743, USA"
}, {
		"location": {
				"latitude": 402647019,
				"longitude": -747071791
		},
		"name": "330 Evelyn Avenue, Hamilton Township, NJ 08619, USA"
}, {
		"location": {
				"latitude": 412567807,
				"longitude": -741058078
		},
		"name": "New York State Reference Route 987E, Southfields, NY 10975, USA"
}, {
		"location": {
				"latitude": 416855156,
				"longitude": -744420597
		},
		"name": "103-271 Tempaloni Road, Ellenville, NY 12428, USA"
}, {
		"location": {
				"latitude": 404663628,
				"longitude": -744820157
		},
		"name": "1300 Airport Road, North Brunswick Township, NJ 08902, USA"
}, {
		"location": {
				"latitude": 407113723,
				"longitude": -749746483
		},
		"name": ""
}, {
		"location": {
				"latitude": 402133926,
				"longitude": -743613249
		},
		"name": ""
}, {
		"location": {
				"latitude": 400273442,
				"longitude": -741220915
		},
		"name": ""
}, {
		"location": {
				"latitude": 411236786,
				"longitude": -744070769
		},
		"name": ""
}, {
		"location": {
				"latitude": 411633782,
				"longitude": -746784970
		},
		"name": "211-225 Plains Road, Augusta, NJ 07822, USA"
}, {
		"location": {
				"latitude": 415830701,
				"longitude": -742952812
		},
		"name": ""
}, {
		"location": {
				"latitude": 413447164,
				"longitude": -748712898
		},
		"name": "165 Pedersen Ridge Road, Milford, PA 18337, USA"
}, {
		"location": {
				"latitude": 405047245,
				"longitude": -749800722
		},
		"name": "100-122 Locktown Road, Frenchtown, NJ 08825, USA"
}, {
		"location": {
				"latitude": 418858923,
				"longitude": -746156790
		},
		"name": ""
}, {
		"location": {
				"latitude": 417951888,
				"longitude": -748484944
		},
		"name": "650-652 Willi Hill Road, Swan Lake, NY 12783, USA"
}, {
		"location": {
				"latitude": 407033786,
				"longitude": -743977337
		},
		"name": "26 East 3rd Street, New Providence, NJ 07974, USA"
}, {
		"location": {
				"latitude": 417548014,
				"longitude": -740075041
		},
		"name": ""
}, {
		"location": {
				"latitude": 410395868,
				"longitude": -744972325
		},
		"name": ""
}, {
		"location": {
				"latitude": 404615353,
				"longitude": -745129803
		},
		"name": ""
}, {
		"location": {
				"latitude": 406589790,
				"longitude": -743560121
		},
		"name": "611 Lawrence Avenue, Westfield, NJ 07090, USA"
}, {
		"location": {
				"latitude": 414653148,
				"longitude": -740477477
		},
		"name": "18 Lannis Avenue, New Windsor, NY 12553, USA"
}, {
		"location": {
				"latitude": 405957808,
				"longitude": -743255336
		},
		"name": "82-104 Amherst Avenue, Colonia, NJ 07067, USA"
}, {
		"location": {
				"latitude": 411733589,
				"longitude": -741648093
		},
		"name": "170 Seven Lakes Drive, Sloatsburg, NY 10974, USA"
}, {
		"location": {
				"latitude": 412676291,
				"longitude": -742606606
		},
		"name": "1270 Lakes Road, Monroe, NY 10950, USA"
}, {
		"location": {
				"latitude": 409224445,
				"longitude": -748286738
		},
		"name": "509-535 Alphano Road, Great Meadows, NJ 07838, USA"
}, {
		"location": {
				"latitude": 406523420,
				"longitude": -742135517
		},
		"name": "652 Garden Street, Elizabeth, NJ 07202, USA"
}, {
		"location": {
				"latitude": 401827388,
				"longitude": -740294537
		},
		"name": "349 Sea Spray Court, Neptune City, NJ 07753, USA"
}, {
		"location": {
				"latitude": 410564152,
				"longitude": -743685054
		},
		"name": "13-17 Stanley Street, West Milford, NJ 07480, USA"
}, {
		"location": {
				"latitude": 408472324,
				"longitude": -740726046
		},
		"name": "47 Industrial Avenue, Teterboro, NJ 07608, USA"
}, {
		"location": {
				"latitude": 412452168,
				"longitude": -740214052
		},
		"name": "5 White Oak Lane, Stony Point, NY 10980, USA"
}, {
		"location": {
				"latitude": 409146138,
				"longitude": -746188906
		},
		"name": "Berkshire Valley Management Area Trail, Jefferson, NJ, USA"
}, {
		"location": {
				"latitude": 404701380,
				"longitude": -744781745
		},
		"name": "1007 Jersey Avenue, New Brunswick, NJ 08901, USA"
}, {
		"location": {
				"latitude": 409642566,
				"longitude": -746017679
		},
		"name": "6 East Emerald Isle Drive, Lake Hopatcong, NJ 07849, USA"
}, {
		"location": {
				"latitude": 408031728,
				"longitude": -748645385
		},
		"name": "1358-1474 New Jersey 57, Port Murray, NJ 07865, USA"
}, {
		"location": {
				"latitude": 413700272,
				"longitude": -742135189
		},
		"name": "367 Prospect Road, Chester, NY 10918, USA"
}, {
		"location": {
				"latitude": 404310607,
				"longitude": -740282632
		},
		"name": "10 Simon Lake Drive, Atlantic Highlands, NJ 07716, USA"
}, {
		"location": {
				"latitude": 409319800,
				"longitude": -746201391
		},
		"name": "11 Ward Street, Mount Arlington, NJ 07856, USA"
}, {
		"location": {
				"latitude": 406685311,
				"longitude": -742108603
		},
		"name": "300-398 Jefferson Avenue, Elizabeth, NJ 07201, USA"
}, {
		"location": {
				"latitude": 419018117,
				"longitude": -749142781
		},
		"name": "43 Dreher Road, Roscoe, NY 12776, USA"
}, {
		"location": {
				"latitude": 412856162,
				"longitude": -745148837
		},
		"name": "Swan Street, Pine Island, NY 10969, USA"
}, {
		"location": {
				"latitude": 416560744,
				"longitude": -746721964
		},
		"name": "66 Pleasantview Avenue, Monticello, NY 12701, USA"
}, {
		"location": {
				"latitude": 405314270,
				"longitude": -749836354
		},
		"name": ""
}, {
		"location": {
				"latitude": 414219548,
				"longitude": -743327440
		},
		"name": ""
}, {
		"location": {
				"latitude": 415534177,
				"longitude": -742900616
		},
		"name": "565 Winding Hills Road, Montgomery, NY 12549, USA"
}, {
		"location": {
				"latitude": 406898530,
				"longitude": -749127080
		},
		"name": "231 Rocky Run Road, Glen Gardner, NJ 08826, USA"
}, {
		"location": {
				"latitude": 407586880,
				"longitude": -741670168
		},
		"name": "100 Mount Pleasant Avenue, Newark, NJ 07104, USA"
}, {
		"location": {
				"latitude": 400106455,
				"longitude": -742870190
		},
		"name": "517-521 Huntington Drive, Manchester Township, NJ 08759, USA"
}, {
		"location": {
				"latitude": 400066188,
				"longitude": -746793294
		},
		"name": ""
}, {
		"location": {
				"latitude": 418803880,
				"longitude": -744102673
		},
		"name": "40 Mountain Road, Napanoch, NY 12458, USA"
}, {
		"location": {
				"latitude": 414204288,
				"longitude": -747895140
		},
		"name": ""
}, {
		"location": {
				"latitude": 414777405,
				"longitude": -740615601
		},
		"name": ""
}, {
		"location": {
				"latitude": 415464475,
				"longitude": -747175374
		},
		"name": "48 North Road, Forestburgh, NY 12777, USA"
}, {
		"location": {
				"latitude": 404062378,
				"longitude": -746376177
		},
		"name": ""
}, {
		"location": {
				"latitude": 405688272,
				"longitude": -749285130
		},
		"name": ""
}, {
		"location": {
				"latitude": 400342070,
				"longitude": -748788996
		},
		"name": ""
}, {
		"location": {
				"latitude": 401809022,
				"longitude": -744157964
		},
		"name": ""
}, {
		"location": {
				"latitude": 404226644,
				"longitude": -740517141
		},
		"name": "9 Thompson Avenue, Leonardo, NJ 07737, USA"
}, {
		"location": {
				"latitude": 410322033,
				"longitude": -747871659
		},
		"name": ""
}, {
		"location": {
				"latitude": 407100674,
				"longitude": -747742727
		},
		"name": ""
}, {
		"location": {
				"latitude": 418811433,
				"longitude": -741718005
		},
		"name": "213 Bush Road, Stone Ridge, NY 12484, USA"
}, {
		"location": {
				"latitude": 415034302,
				"longitude": -743850945
		},
		"name": ""
}, {
		"location": {
				"latitude": 411349992,
				"longitude": -743694161
		},
		"name": ""
}, {
		"location": {
				"latitude": 404839914,
				"longitude": -744759616
		},
		"name": "1-17 Bergen Court, New Brunswick, NJ 08901, USA"
}, {
		"location": {
				"latitude": 414638017,
				"longitude": -745957854
		},
		"name": "35 Oakland Valley Road, Cuddebackville, NY 12729, USA"
}, {
		"location": {
				"latitude": 412127800,
				"longitude": -740173578
		},
		"name": ""
}, {
		"location": {
				"latitude": 401263460,
				"longitude": -747964303
		},
		"name": ""
}, {
		"location": {
				"latitude": 412843391,
				"longitude": -749086026
		},
		"name": ""
}, {
		"location": {
				"latitude": 418512773,
				"longitude": -743067823
		},
		"name": ""
}, {
		"location": {
				"latitude": 404318328,
				"longitude": -740835638
		},
		"name": "42-102 Main Street, Belford, NJ 07718, USA"
}, {
		"location": {
				"latitude": 419020746,
				"longitude": -741172328
		},
		"name": ""
}, {
		"location": {
				"latitude": 404080723,
				"longitude": -746119569
		},
		"name": ""
}, {
		"location": {
				"latitude": 401012643,
				"longitude": -744035134
		},
		"name": ""
}, {
		"location": {
				"latitude": 404306372,
				"longitude": -741079661
		},
		"name": ""
}, {
		"location": {
				"latitude": 403966326,
				"longitude": -748519297
		},
		"name": ""
}, {
		"location": {
				"latitude": 405002031,
				"longitude": -748407866
		},
		"name": ""
}, {
		"location": {
				"latitude": 409532885,
				"longitude": -742200683
		},
		"name": ""
}, {
		"location": {
				"latitude": 416851321,
				"longitude": -742674555
		},
		"name": ""
}, {
		"location": {
				"latitude": 406411633,
				"longitude": -741722051
		},
		"name": "3387 Richmond Terrace, Staten Island, NY 10303, USA"
}, {
		"location": {
				"latitude": 413069058,
				"longitude": -744597778
		},
		"name": "261 Van Sickle Road, Goshen, NY 10924, USA"
}, {
		"location": {
				"latitude": 418465462,
				"longitude": -746859398
		},
		"name": ""
}, {
		"location": {
				"latitude": 411733222,
				"longitude": -744228360
		},
		"name": ""
}, {
		"location": {
				"latitude": 410248224,
				"longitude": -747127767
		},
		"name": "3 Hasta Way, Newton, NJ 07860, USA"
}]
"""

def read_route_guide_database(): 
	feature_list = []
	for item in json.loads(_jsondb):
		feature = route_guide_pb2.Feature(
				name=item["name"],
				location=route_guide_pb2.Point(
						latitude=item["location"]["latitude"],
						longitude=item["location"]["longitude"]))
		feature_list.append(feature)
	return feature_list

db = read_route_guide_database ()	


if __name__ == "__main__":
	import skitai
	skitai.run (
		address = "0.0.0.0",
		port = 5000,
		mount = ("/routeguide.RouteGuide", app)
	)
	



from skitai.saddle import Saddle
import skitai

app = Saddle (__name__)

app.debug = True
app.use_reloader = True
app.jinja_overlay ()

@app.route ("/echo")
def echo (was, message, client_id, event):
	if was.wsinit ():
		#return was.wsconfig (skitai.WS_SIMPLE, 60, skitai.WS_MSG_JSON)			
		return was.wsconfig (skitai.WS_SIMPLE, 60)
	if event == skitai.WS_EVT_ENTER:
		return "Welcome Client %s" % client_id
	if event: # ignore the other events
		return
				
	was.websocket.send ("You said," + message)
	was.websocket.send ("acknowledge")	

@app.route ("/talk")
def talk (was, name = "Hans Roh"):
	if was.wsinit ():
		return was.wsconfig (skitai.WS_DEDICATE, 60)		
	
	while 1:
		m = was.websocket.getwait (10)
		if m is None:
			break		
		if m.lower () == "bye":
			was.websocket.send ("Bye, have a nice day." + m)
			was.websocket.close ()
			break
		elif m.lower () == "hello":
			was.websocket.send ("Hello, " + name)				
		else:	
			was.websocket.send ("You Said:" + m)

@app.route ("/chat")
def chat (was, message, client_id, event, room_id):
	if was.wsinit ():
		return was.wsconfig (skitai.WS_GROUPCHAT, 60)
	if event == skitai.WS_EVT_ENTER:
		return "Client %s has entered" % client_id
	if event == skitai.WS_EVT_EXIT:
		return "Client %s has leaved" % client_id
	return "Client %s Said: %s" % (client_id, message)
			
@app.route ("/")
def websocket (was, mode = "echo"):
	if mode == "talk":
		mode += "?name=Hans"
	elif mode == "chat":	
		mode += "?room_id=1"
	return was.render ("websocket.html", path = mode)
	
if __name__ == "__main__":
	import skitai
	skitai.run (
		address = "0.0.0.0",
		port = 5000,
		mount = ("/websocket", app)
	)
	


"""
Hans Roh 2015 -- http://osp.skitai.com
License: BSD
"""
import re
import sys
import os
import shutil, glob
from distutils.core import setup
from setuptools.command.easy_install import easy_install

class easy_install_default(easy_install):
	def __init__(self):
		from distutils.dist import Distribution		
		self.distribution = Distribution()
		self.initialize_options()

e = easy_install_default()
try: e.finalize_options()
except: pass
python_package_dir = e.install_dir

def mkdir (tdir, mod = -1):
	while tdir:
		if tdir [-1] in ("\\/"):
			tdir = tdir [:-1]
		else:
			break	

	if os.path.isdir (tdir): return	
	chain = [tdir]	
	while 1:
		tdir, last = os.path.split (tdir)		
		if not last: 
			break
		if tdir:
			chain.insert (0, tdir)
	
	for dir in chain [1:]:
		try: 
			os.mkdir (dir)
			if os.name == "posix" and mod != -1:
				os.chmod (dir, mod)				
		except OSError as why:
			if why.errno in (17, 183): continue
			else: raise
				
if sys.argv[-1] == 'publish':
	if os.name == "nt":
		os.system('python setup.py sdist upload') # bdist_wininst --target-version=2.7
	else:		
		os.system('python setup.py sdist upload')
	sys.exit()

classifiers = [
  'License :: OSI Approved :: BSD License',
  'Development Status :: 4 - Beta',
  'Topic :: Internet :: WWW/HTTP',
	'Topic :: Internet :: WWW/HTTP :: HTTP Servers',				
	'Environment :: Console',
	'Environment :: No Input/Output (Daemon)',
	'Topic :: Software Development :: Libraries :: Python Modules',
	'Intended Audience :: Developers',
	'Intended Audience :: Science/Research',
	'Programming Language :: Python',
	'Programming Language :: Python :: 2.7',
	'Programming Language :: Python :: 3'
]


packages = ['skitaid']
package_dir = {'skitaid': 'skitaid'}

skitaid_files = [
	"bin/*.py",
	"etc/init/skitaid.conf",
	"etc/init.d/skitaid", 
	"etc/skitaid/skitaid.conf",
	"etc/skitaid/servers-available/README.txt", 
	"etc/skitaid/servers-enabled-posix/example.conf",
	"etc/skitaid/servers-enabled-nt/example.conf",
	"etc/skitaid/certifications/*.*",
	"wsgi/example/*.py",
	"wsgi/example/static/*.*",	
	"wsgi/example/resources/*.*",	
	"wsgi/example/templates/*.*",
	"wsgi/example/apppackages/*.*",
]

package_data = {
	"skitaid": skitaid_files
}

examples = [ 
	os.path.join ("skitaid/wsgi/example", each) for each in (
		"app.py", "gateway.py", "grpc_route_guide.py", "proxy.py", "reverse_proxy.py", "rpc2.py", 
		"tls12.py", "websocket.py", "websocket-flask.py", "auth.py"
	)
]

templates = [ 
	os.path.join ("skitaid/wsgi/example/templates", each) for each in (
		"index.html", "documentation.html", "websocket.html", "websocket-flask.html"
	)
]

apppackages = [ 
	os.path.join ("skitaid/wsgi/example/apppackages", each) for each in (
		"__init__.py", "route_guide_pb2.py"
	)
]

							
if os.name == "nt":
	if not os.path.isfile ("c:\\skitaid\\etc\\skitaid.conf"):
		mkdir ("c:\\skitaid\\etc\\servers-enabled")	
		shutil.copyfile ("skitaid/etc/skitaid/skitaid.conf", "c:\\skitaid\\etc\\skitaid.conf")
		shutil.copyfile ("skitaid/etc/skitaid/servers-enabled-nt/example.conf", "c:\\skitaid\\etc\\servers-enabled\\example.conf")
	else:
		mkdir ("c:\\skitaid\\etc\\servers-available")
		shutil.copyfile ("skitaid/etc/skitaid/servers-enabled-nt/example.conf", "c:\\skitaid\\etc\\servers-available\\example.conf")
			
	data_files = [
		("c:\\skitaid\\etc\\certifications", ["skitaid/etc/skitaid/certifications/README.txt", "skitaid/etc/skitaid/certifications/example.pem", "skitaid/etc/skitaid/certifications/example.key"]),
		("c:\\skitaid\\etc\\servers-available", ["skitaid/etc/skitaid/servers-available/README.txt"]),
		("c:\\skitaid\\bin", ["skitaid/bin/install_win32_service.py", "skitaid/bin/skitaid.py", "skitaid/bin/skitaid-instance.py", "skitaid/bin/skitaid-smtpda.py", "skitaid/bin/skitaid-cron.py"]),
		(
			"c:\\skitaid\\wsgi\\example", examples	
		),
		("c:\\skitaid\\wsgi\\example\\static", ["skitaid/wsgi/example/static/reindeer.jpg"]),
		("c:\\skitaid\\wsgi\\example\\resources", ["skitaid/wsgi/example/resources/sqlite3.db"]),
		("c:\\skitaid\\wsgi\\example\\apppackages", apppackages),
		("c:\\skitaid\\wsgi\\example\\templates", templates)		
	]
	
else:
	if not os.path.isfile ("/etc/skitaid/skitaid.conf"):
		mkdir ("/etc/skitaid/servers-enabled")
		shutil.copyfile ("skitaid/etc/skitaid/skitaid.conf", "/etc/skitaid/skitaid.conf")
		shutil.copyfile ("skitaid/etc/skitaid/servers-enabled-posix/example.conf", "/etc/skitaid/servers-enabled/example.conf")
	else:
		mkdir ("/etc/skitaid/servers-available")
		shutil.copyfile ("skitaid/etc/skitaid/servers-enabled-posix/example.conf", "/etc/skitaid/servers-available/example.conf")	
		
	for fn in ["skitaid.py", "skitaid-instance.py", "skitaid-smtpda.py", "skitaid-cron.py"]:
		source = os.path.join ("skitaid/bin/", fn)
		target = os.path.join ("/usr/local/bin", fn)
		
		exists_python = "#!/usr/bin/python%s\n" % sys.version_info [0]		
		exists_python = exists_python.encode ("utf8")
		
		with open (source, "rb") as f:
			source_python = f.readline ()				
			data = f.read ()			
		with open (target, "wb") as f:
			f.write (exists_python)
			f.write (data)
		os.chmod (target, 0o755)
		
	os.chmod ("skitaid/etc/init.d/skitaid", 0o755)			
	data_files = [		
		("/etc/skitaid/certifications", ["skitaid/etc/skitaid/certifications/README.txt", "skitaid/etc/skitaid/certifications/example.pem", "skitaid/etc/skitaid/certifications/example.key"]),
		("/etc/skitaid/servers-available", ["skitaid/etc/skitaid/servers-available/README.txt"]),		
		("/var/wsgi/example", examples),
		("/var/wsgi/example/static", ["skitaid/wsgi/example/static/reindeer.jpg"]),
		("/var/wsgi/example/resources", ["skitaid/wsgi/example/resources/sqlite3.db"]),
		("/var/wsgi/example/apppackages", apppackages),
		("/var/wsgi/example/templates", templates),
		("/etc/init.d", ["skitaid/etc/init.d/skitaid"])		
	]
	

if not os.path.isdir ("skitaid"):
	new_data_files = []
	for target, files in data_files:
		newfiles = []
		for each in files:
			newfiles.append (os.path.join (python_package_dir, each))
		new_data_files.append ((target, newfiles))
	data_files = new_data_files

with open('skitaid/__init__.py', 'r') as fd:
	version = re.search(r'^__version__\s*=\s*"(.*?)"',fd.read(), re.M).group(1)
		
setup(
	name='skitaid',
	version=version,
	description='Skitai WSGI App Engine',	
	url = 'https://gitlab.com/hansroh/skitaid',
	author='Hans Roh',
	author_email='hansroh@gmail.com',	
	packages=packages,
	package_dir=package_dir,
	data_files = data_files,
	package_data = package_data,
	license='BSD',
	platforms = ["posix", "nt"],
	download_url = "https://pypi.python.org/pypi/skitaid",
	install_requires = ["skitai>=0.24.3"],
	classifiers=classifiers
)




#!/usr/bin/python3
# 2015. 11. 27 by Hans Roh hansroh@gmail.com

import sys, os, getopt
from skitai import lifetime
from aquests.lib import flock, pathtool, logger
from aquests.lib.athreads import select_trigger
from aquests.protocols.smtp import async_smtp, composer
import signal
import time
import glob

def hTERM (signum, frame):			
	lifetime.shutdown (0, 1.0)

def hQUIT (signum, frame):			
	lifetime.shutdown (0, 30.0)

def hHUP (signum, frame):			
	lifetime.shutdown (3, 30.0)


class	SMTPDeliverAgent:
	CONCURRENTS = 4
	MAX_RETRY = 10
	UNDELIVERS_KEEP_MAX = 2592000

	def __init__ (self, config, logpath, varpath, consol):
		self.config = config
		self.logpath = logpath
		self.varpath = varpath
		self.consol = consol
		self.last_maintern = 0
		self.que = {}
		self.actives = {}
		self.flock = None
		self.shutdown_in_progress = False
		
		self.setup ()
	
	def maintern_shutdown_request (self, now):
		req = self.flock.lockread ("signal")
		if not req: return
		self.logger ("[info] got signal - %s" % req)
		if req == "terminate":			
			lifetime.shutdown (0, 1.0)
		elif req == "restart":			
			lifetime.shutdown (3, 30.0)		
		elif req == "shutdown":
			lifetime.shutdown (0, 30.0)		
		elif req == "rotate":
			try: self.logger.rotate ()
			except: self.logger.trace ()
		else:
			self.logger ("[error] unknown signal - %s" % req)
		self.flock.unlock ("signal")
	
	def clean_shutdown_control (self, phase, time_in_this_phase):
		if phase == 1:
			self.shutdown_in_progress = True
	
	def close (self):
		self.logger ("[info] service smtpda stopped")
						
	def run (self):
		self.logger ("[info] service smtpda started")
		try:
			try:
				lifetime.loop (3.0)
			except:
				self.logger.trace ()
		finally:
			self.close ()		
					
	def setup (self):
		val = self.config.getint ("smtpda", "max_retry", 10)
		if val: self.MAX_RETRY = val
		val = self.config.getint ("smtpda", "undelivers_keep_max_days", 30)
		if val: self.UNDELIVERS_KEEP_MAX = val * 3600 * 24

		self.default_smtp = (
			self.config.getopt ("smtpda", "smtpserver"),
			self.config.getopt ("smtpda", "user"),
			self.config.getopt ("smtpda", "password"),
			self.config.getopt ("smtpda", "ssl") in ("1", "yes") and True or False
		)
		
		# dummy file like object for keeping lifetime loop
		if os.name == "nt":
			select_trigger.trigger.address = ('127.9.9.9', 19998)
		select_trigger.trigger ()
		
		self.path_spool = os.path.join (self.varpath, "mail", "spool")
		pathtool.mkdir (self.path_spool)
		composer.Composer.SAVE_PATH = self.path_spool
		
		self.path_undeliver = os.path.join (self.varpath, "mail", "undeliver")
		pathtool.mkdir (self.path_undeliver)
		
		if self.consol:
			self.logger = logger.screen_logger ()
		else:
			self.logger = logger.rotate_logger (self.logpath, "smtpda", "daily")
		
		lifetime.init ()		
		lifetime.maintern.sched (3.0, self.handle_spool)
		if os.name == "nt":
			self.flock = flock.Lock (os.path.join (self.varpath, "lock"))
			self.flock.unlockall ()
			lifetime.maintern.sched (10.0, self.maintern_shutdown_request)
		
		if os.name != "nt":
			def hUSR1 (signum, frame):	
				self.logger.rotate ()	
			
			signal.signal(signal.SIGUSR1, hUSR1)
			signal.signal(signal.SIGHUP, hHUP)
			signal.signal(signal.SIGTERM, hTERM)
			signal.signal(signal.SIGQUIT, hQUIT)		
	
	def send (self):
		if self.shutdown_in_progress:
			return
			
		while self.que:
			fn = self.que.popitem () [0]
			if fn in self.actives: continue

			self.actives [fn] = time.time ()
			path = os.path.join (self.path_spool, fn)
			cmps = composer.load (path)
			if cmps.get_SMTP () is None:
				cmps.set_smtp (*self.default_smtp)
			if cmps.is_SSL ():
				request = async_smtp.SMTP_SSL
			else:	
				request = async_smtp.SMTP
			request (cmps, self.logger, self.when_done)					
			break
	
	def when_done (self, cmps, code, reps):
		fn = os.path.split (cmps.get_FILENAME ()) [-1]
		
		if code == -250:
			cmps.remove ()
		else:
			if cmps.get_RETRYS () > self.MAX_RETRY:
				cmps.moveto (self.path_undeliver)
			else:	
				cmps.moveto (self.path_spool)		
		del self.actives [fn]
		self.send ()
	
	def maintern (self, current_time):
		for path in glob.glob (os.path.join (self.path_undeliver, "*.*")):
			mtime = os.path.getmtime (path)
			if mtime + self.UNDELIVERS_KEEP_MAX > current_time: # over a month
				try: os.remove (path)
				except: self.logger.trace ()
		self.last_maintern = time.time ()
		
	def handle_spool (self, current_time):
		if self.shutdown_in_progress:
			return
		
		if self.last_maintern + 3600 >  current_time:
			self.maintern (current_time)
		
		if not self.que: # previous que has priority			
			for path in glob.glob (os.path.join (self.path_spool, "*.*")):
				try:
					fn = os.path.split (path)[-1]
					retrys, digest = fn.split (".")
					assert (len (digest) == 32)
					retrys = int (retrys)
				except (ValueError, AssertionError):
					continue
				except:
					self.logger.trace ()
					continue
						
				mtime = os.path.getmtime (path)
				if retrys > 7:
					delta = 21600
				elif retrys == 0:
					delta = 0
				else:
					delta = 4 ** retrys
				
				if retrys >= self.MAX_RETRY:
					self.que [fn] = None
				elif mtime + delta <= current_time and fn not in self.que and fn not in self.actives:
					self.que [fn] = None
		
		for i in range (self.CONCURRENTS - len (self.actives)):
			self.send ()
	
		
def usage ():
		print("""
Usage:
	skitaid-smtpda.py [options...]

Options:
	--log or -l: print log
	--verbose or -v
	--help or -h

Examples:
	ex. skitaid-smtpda.py -l
	ex. skitaid-smtpda.py -v
	""")


if __name__ == "__main__":
	argopt = getopt.getopt(sys.argv[1:], "hvl", ["help", "verbose", "log"])
	_varpath = None
	_consol = False
	_log = False
	
	for k, v in argopt [0]:
		if k == "--log" or k == "-l":
			_log = True
		elif k == "--verbose" or k == "-v":	
			_consol = True
		elif k == "--help" or k == "-h":	
			usage ()
			sys.exit ()
	
	import skitaid
	_config = skitaid.cf
	_varpath = os.path.join (skitaid.VARDIR, "daemons", "smtpda")
	_logpath = os.path.join (skitaid.LOGDIR, "daemons", "smtpda")
	
	if _log:		
		skitaid.printlog (os.path.join (_logpath, "smtpda.log"))
		sys.exit (0)
		
	lck = flock.Lock (os.path.join (_varpath, "lock"))
	pidlock = lck.get_pidlock ()
	
	if pidlock.isalive ():
		print("[error] already running")
		sys.exit (1)
	
	pathtool.mkdir (_logpath)
	if not _consol: # service mode
		from aquests.lib import devnull		
		sys.stdout = devnull.devnull ()		
		sys.stderr = open (os.path.join (_logpath, "stderr.log"), "a")
	
	pidlock.make ()
	service = SMTPDeliverAgent (_config, _logpath, _varpath, _consol)
		
	try:
		service.run ()		
	finally:	
		pidlock.remove ()
		if not _consol:
			sys.stderr.close ()	
		sys.exit (lifetime._exit_code)
		



from skitai.saddle import Saddle
import time, math
import json
from apppackages import route_guide_pb2

app = Saddle (__name__)
app.debug = True
app.use_reloader = True

@app.route ("/")
def RouteChat (was):
	return "<h1>Reverse Proxing</h1><a href='/lb/proxed'>Click Here</a>"

@app.route ("/proxed")
def RouteChat (was):
	return "<h1>Reverse Proxed</h1><a href='/lb/'>Click Here</a>"
	
if __name__ == "__main__":
	import skitai
	skitai.run (		
		clusters = {"@pypi": ("https", "pypi.python.org")},
		mount = [
			("/", app),
			("/lb", "@pypi"),
		]
	)

